@@ -1,5 +1,225 @@
 /* libmondo-devices.c                 Subroutines for handling devices
-   $Id: libmondo-devices.c 1988 2008-06-20 05:50:29Z bruno $
+   $Id: libmondo-devices.c 1316 2007-04-16 14:50:28Z bruno $
+.
+
+
+01/07/2005
+- sensibly_set_tmpdir_and_scratchdir() --- exclude smb and smbfs and (new) cifs
+
+10/23
+- if root mounted at '/dev/rd/' then say FALSE, not mounted on ramdisk
+  just to please Fred Beondo :-p
+
+07/25
+- updated get_phys_size_of_drive() to support newer, larger drives
+
+07/18
+- better support of users who boot from LVM CD and nuke-restore non-LVM backups
+
+07/08
+- BLKGETSIZE64 instead of BLKGETIO..whatever (Joshua)
+- amended to use BLKGETSIZE if BLKGETSIZE64 missing (Hugo)
+- resort to HDIO_GETGEO if BLKGETSIZE and BLKGETSIZE63 fail
+
+
+06/25
+- added Mandrake 9.2 support to which_boot_loader()
+- cleaned up find_cdrom_device()
+
+06/23
+- rewrote resolve_softlinks_to_get_to_actual_device_file() in C
+
+06/17
+- don't use star automatically if SELINUX detected; let user do that
+
+05/07
+- usage of parted2fdisk instead of fdisk alone (ia32/ia64 compatibility)
+  BCO
+
+04/17
+- replaced INTERNAL_TAPE_BLK_SIZE with bkpinfo->internal_tape_block_size
+
+04/13
+- if kernel >= 2.6 then ask for /dev entry, not SCSI node
+
+04/12
+- in mount_CDROM_...whatever, don't use bkpinfo_DONTUSETHIS to see if
+  mounting a real CD device; look for /dev/ at start of $device instead
+
+04/04
+- when testing for tape drive, use INTERNAL_TAPE_BLK_SIZE, not TAPE_BLOCK_SIZE
+
+04/03
+- don't eject/retract drive if ISO
+
+02/23/2004
+- changed strstr() to strncmp() in a few places
+
+12/11/2003
+- if we can't find the DVD using dvdrecord then try it w/cdrecord
+
+11/15
+- changed a few []s to char*s
+- better support of multiple CD-ROM drives
+
+11/14
+- better find_cdrom_device(), to cope w/ multiple CD writers
+
+10/26
+- call 'mt -f %s offline' to eject tape drive in eject_device()
+
+10/25
+- ask user to confirm which NFS share to mount
+- improved which_boot_loader()
+
+10/22
+- tweaked find_cdrom_device() and find_cdrw_device()
+- created find_dvd_device()
+
+10/21
+- changed "/mnt/cdrom" to MNT_CDROM
+
+09/28
+- interactive gui no longer asks dvd's speed
+
+09/26
+- interactive gui now supports dvd
+
+09/23
+- malloc/free global strings in new subroutines - malloc_libmondo_global_strings()
+  and free_libmondo_global_strings() - which are in libmondo-tools.c
+
+09/21
+- ask for CD size when backing up interactively
+- offer to exclude NFS shares if they're present
+
+09/20
+- exclude /dev/shm from sensibly_*()
+
+09/18
+- less verbose logging when insisting on CD #n
+
+09/09
+- added resolve_softlinks_to_get_to_actual_device_file()
+
+09/05
+- rewrote inisist_on_this_cd_number() and what_number_cd_is_this()
+
+09/02
+- is_this_a_valid_disk_format() -- treat ntfs (a.k.a. type 7) as recognized format
+  just in case user's backup includes a partimagehack-imaged drive
+
+08/01 - 08/31
+- better boot-time tape detection
+- remove backup-time, erroneous, "remove floppy" msg
+- working on am_I_in_disaster_recovery_mode()
+- interactively_obtain_...() - pause & ask user to remove CD/floppy
+
+06/01 - 07/31
+- in which_boot_loader(), also search /dev/cciss for boot loader
+- fixed calls to popup_and_get_string()
+- fixed bug in get_phys_size_of_drive()
+- fixed bug in where_is_root_mounted()
+- commented out a spurious assert()
+
+05/01 - 05/31
+- superior get_phys_size_of_drive() (Joshua Oreman)
+- fixed call to inject_device() --- Andree L.
+- find_cdrom_device() now may, if asked to, mount drive if possible,
+  to test that it has a CD in it; it then unmounts
+- mount_CDROM_here() now calls retract_cd_and_defeat_autorun() as well
+- fixed support for subdir-within-NFS-mount
+- cleaned up some FreeBSD-specific stuff
+- added Joshua Oreman's FreeBSD patches
+- mount_CDROM_here() --- mkdir -p mountpt
+- add older drive support to get_phys_size_of_drive()
+
+04/01 - 04/30
+- made which_boot_loader() more friendly towards Red Hat 9
+- find_and_mount_actual_cd() retracts all CD trays
+- added lots of log_OS_error()'s and assert()'s
+- fix ISO support in interactively_restore_...()
+- made find_cdrom_device() skip dmesg if second-stage string is empty
+- in find_cdrw_device(), find cdrecord before dvdrecord
+- fixed am_i_in_disaster_recovery_mode()
+- fixed insist_on_this_cd_number()
+
+03/01 - 03/31
+- more work on find_cdrom_device()
+- find_cdrom_device() --- if nonexistent/not found then
+  make sure to return '' as dev str and 1 as res
+- cleaner find_cdrom_device(), to find DVD instead of CD-ROM
+  as last resort
+- fix insist_on_this_cd_number()
+- add user the speed of CD writer, if writing
+- if root is /dev/root then assume not a ramdisk
+
+01/07 - 02/28
+- fixed find_cdrom_device (Stan Benoit)
+- if root is at /dev/root/root then LVM (not ramdisk)
+- fix am_I_in_disaster_recovery_mode() to recognize that /dev/rd/cp*
+  entries are not ramdisks but RAID disks
+- find_cdrw_device() no longer complains to stderr if no cdrecord
+
+01/02/2003
+- sensibly_set_tmpdir_and_scratchdir() --- exclude smb and smbfs
+
+12/01/2002
+- don't ask for isodir path if in disaster recovery mode
+
+11/01 - 11/30
+- added g_restoring_live_from_cd;
+- handle_incoming_parameters() will sensibly set tmpdir and scratchdir
+- properly wipe spurious scratchdir and tmpdir
+- modified sensibly_...() to use tmp.mondo.* and mondo.scratch.*
+  instead of mondo.tmpdir.* and mondo.scratchdir.*
+- line 861,912: set media_size[0] to 1999 as well as 1 thru N
+- chmod 700, not 770, in make_fifo()
+- sensibly_set_tmpdir_and_scratchdir() will now prefix a '/' to the
+  strings if it is missing
+
+10/01 - 10/31
+- changed find_cdrom_device() to ignore DVD drives in the hd[a-d] search;
+  consequently, the subroutine will try scd0, sr0, etc. afterwards
+- commented code
+- fixed obscure bug in find_and_mount_actual_cd()
+- code which sensibly configures tmpdir,scratchdir is now sep. sub.
+- changed sort -s to sort -n <-- sensibly...()
+
+09/01 - 09/30
+- added list_of_NFS_devices_and_mounts()
+- try /dev/st0 if no tape specified at all
+- change '64k' to TAPE_BLOCK_SIZE
+- improved find_cdrom_device()
+- if cdrecord not found then find_cdrom_device() returns 1
+- if disaster recovery mode then don't ask for CD-ROM's /dev entry;
+  assume /dev/cdrom
+- if restoring data then ask for CD-ROM's /dev entry (not its SCSI node)
+- run_program_and_log_output() now takes boolean operator to specify
+  whether it will log its activities in the event of _success_
+- changed sort -n +4 to sort -s +4 (Troff)
+- ask if user's kernel is sane; if 'no' then use FAILSAFE
+- ask user to confirm the tape/CD device name
+- if restoring (not archiving) then don't try to find CD-ROM's SCSI
+  node; try to find its /dev entry instead
+- better detection of biggest partition
+
+08/01 - 08/31
+- fixed verify bug --- CD#1 was being verified again & again & ...
+- detect boot loader + device; if not detectible then _ask_
+- if run interactively, assume tape size is irrelevant & don't ask
+  for it; Mondo should handle end-of-tape gracefully now
+- worked on handling unknowng media size
+- added interactively_obtain_media_parameters_from_user()
+- when trying to mount CD-ROM, find_and_mount_actual_cd() will
+  now call find_cdrom_device() to try to find the /dev entry first
+- fixed serious bug in zero_out_a_device()
+- cleaned up some log_it() calls
+- fixed minor bug in find_cdrom_device()
+- better at distinguishing between live filesystem and ramdisk
+
+07/24/2002
+- created
 */
 
 /**
@@ -31,17 +251,17 @@
 #endif
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-devices.c 1988 2008-06-20 05:50:29Z bruno $";
+//static char cvsid[] = "$Id: libmondo-devices.c 1316 2007-04-16 14:50:28Z bruno $";
 
 extern int g_current_media_number;
 extern double g_kernel_version;
 
 extern bool g_ISO_restore_mode;
+extern struct s_bkpinfo *g_bkpinfo_DONTUSETHIS;
+extern char *g_erase_tmpdir_and_scratchdir;
 extern char *g_selfmounted_isodir;
 extern char *MONDO_LOGFILE;
 
-extern void setup_tmpdir(char *path);
-
 static char g_cdrw_drive_is_here[MAX_STR_LEN / 4] = "";
 static char g_cdrom_drive_is_here[MAX_STR_LEN / 4] = "";
 static char g_dvd_drive_is_here[MAX_STR_LEN / 4] = "";
@@ -52,17 +272,13 @@ static char g_dvd_drive_is_here[MAX_STR_LEN / 4] = "";
  * @ingroup globalGroup
  */
 bool g_restoring_live_from_cd = FALSE;
-bool g_restoring_live_from_nfs = FALSE;
 
 extern t_bkptype g_backup_media_type;	// set by main()
 
-/* Reference to global bkpinfo */
-extern struct s_bkpinfo *bkpinfo;
 
 
 
-
-void set_g_cdrom_and_g_dvd_to_bkpinfo_value()
+void set_g_cdrom_and_g_dvd_to_bkpinfo_value(struct s_bkpinfo *bkpinfo)
 {
 	strcpy(g_cdrom_drive_is_here, bkpinfo->media_device);	// just in case
 	strcpy(g_dvd_drive_is_here, bkpinfo->media_device);	// just in case
@@ -133,7 +349,8 @@ bool am_I_in_disaster_recovery_mode(void)
 #endif
 
 	if (is_this_a_ramdisk) {
-		if (!does_file_exist("/THIS-IS-A-RAMDISK")) {
+		if (!does_file_exist("/THIS-IS-A-RAMDISK")
+			&& !does_file_exist("/tmp/mountlist.txt.sample")) {
 			log_to_screen
 				("Using /dev/root is stupid of you but I'll forgive you.");
 			is_this_a_ramdisk = FALSE;
@@ -186,9 +403,6 @@ static char *bkptype_to_string(t_bkptype bt)
 	case udev:
 		strcpy(output, "udev");
 		break;
-	case usb:
-		strcpy(output, "usb");
-		break;
 	default:
 		strcpy(output, "default");
 	}
@@ -449,7 +663,7 @@ bool does_string_exist_in_first_N_blocks(char *dev, char *str, int n)
  * @return 0 for success, nonzero for failure.
  * @see mount_CDROM_here
  */
-int find_and_mount_actual_cd(char *mountpoint)
+int find_and_mount_actual_cd(struct s_bkpinfo *bkpinfo, char *mountpoint)
 {
 	/*@ buffers ***************************************************** */
 
@@ -501,6 +715,12 @@ int find_and_mount_actual_cd(char *mountpoint)
 
 
 
+/**
+ * Locate a CD-R/W writer's SCSI node.
+ * @param cdrw_device SCSI node will be placed here.
+ * @return 0 for success, nonzero for failure.
+ */
+
 
 /**
  * Locate a CD-R/W writer's SCSI node.
@@ -634,7 +854,7 @@ int find_cdrom_device(char *output, bool try_to_mount)
 		goto end_of_find_cdrom_device;
 	}
 
-	sprintf(mountpoint, "%s/cd.mnt", bkpinfo->tmpdir);
+	sprintf(mountpoint, "/tmp/cd.%d", (int) (random() % 32767));
 	make_hole_for_dir(mountpoint);
 
 	if (find_home_of_exe("cdrecord")) {
@@ -1392,51 +1612,6 @@ int kick_vn(char *dname)
 #endif
 
 
-/**
- * Mount the CD-ROM at @p mountpoint.
- * @param device The device (or file if g_ISO_restore_mode) to mount.
- * @param mountpoint The place to mount it.
- * @return 0 for success, nonzero for failure.
- */
-int mount_USB_here(char *device, char *mountpoint)
-{
-	/*@ buffer ****************************************************** */
-	char *command;
-	char *dev;
-	int retval;
-
-	malloc_string(command);
-	malloc_string(dev);
-	assert_string_is_neither_NULL_nor_zerolength(device);
-	assert_string_is_neither_NULL_nor_zerolength(mountpoint);
-
-	make_hole_for_dir(mountpoint);
-	if (isdigit(device[0])) {
-		return(1);
-	} else {
-		strcpy(dev, device);
-	}
-	log_msg(4, "(mount_USB_here --- device=%s, mountpoint=%s", device,
-			mountpoint);
-
-#ifdef __FreeBSD__
-	sprintf(command, "mount_vfat %s %s 2>> %s",
-			device, mountpoint, MONDO_LOGFILE);
-
-#else
-	sprintf(command, "mount %s -t vfat %s 2>> %s",
-			device, mountpoint, MONDO_LOGFILE);
-#endif
-
-	log_msg(4, command);
-	retval = system(command);
-	log_msg(1, "system(%s) returned %d", command, retval);
-
-	paranoid_free(command);
-	paranoid_free(dev);
-	return (retval);
-}
-
 /**
  * Mount the CD-ROM at @p mountpoint.
  * @param device The device (or file if g_ISO_restore_mode) to mount.
@@ -1522,7 +1697,7 @@ int mount_CDROM_here(char *device, char *mountpoint)
  * @param cd_number_i_want The CD number to ask for.
  */
 void
-insist_on_this_cd_number(int cd_number_i_want)
+insist_on_this_cd_number(struct s_bkpinfo *bkpinfo, int cd_number_i_want)
 {
 
 	/*@ int ************************************************************* */
@@ -1556,20 +1731,19 @@ insist_on_this_cd_number(int cd_number_i_want)
 		if (is_this_device_mounted(MNT_CDROM)) {
 			run_program_and_log_output("umount " MNT_CDROM, 5);
 		}
-		sprintf(tmp, "mkdir -p %s/isodir &> /dev/null", bkpinfo->tmpdir);
-		system(tmp);
+		system("mkdir -p /tmp/isodir &> /dev/null");
 		sprintf(tmp, "%s/%s/%s-%d.iso", bkpinfo->isodir,
 				bkpinfo->nfs_remote_dir, bkpinfo->prefix,
 				cd_number_i_want);
 		if (!does_file_exist(tmp)) {
-			sprintf(tmp, "%s/isodir/%s/%s-%d.iso", bkpinfo->tmpdir,
+			sprintf(tmp, "/tmp/isodir/%s/%s-%d.iso",
 					bkpinfo->nfs_remote_dir, bkpinfo->prefix,
 					cd_number_i_want);
 			if (does_file_exist(tmp)) {
 				log_msg(1,
-						"FIXME - hacking bkpinfo->isodir from '%s' to %s/isodir",
-						bkpinfo->isodir, bkpinfo->tmpdir);
-				sprintf(bkpinfo->isodir, "%s/isodir", bkpinfo->tmpdir);
+						"FIXME - hacking bkpinfo->isodir from '%s' to /tmp/isodir",
+						bkpinfo->isodir);
+				strcpy(bkpinfo->isodir, "/tmp/isodir");
 			}
 		}
 		log_msg(3, "Mounting %s at %s", tmp, MNT_CDROM);
@@ -1579,7 +1753,7 @@ insist_on_this_cd_number(int cd_number_i_want)
 //    g_current_media_number = cd_number_i_want;
 //    return;
 	}
-	if ((res = what_number_cd_is_this()) != cd_number_i_want) {
+	if ((res = what_number_cd_is_this(bkpinfo)) != cd_number_i_want) {
 		log_msg(3, "Currently, we hold %d but we want %d", res,
 				cd_number_i_want);
 		sprintf(tmp, "Insisting on %s #%d",
@@ -1589,7 +1763,7 @@ insist_on_this_cd_number(int cd_number_i_want)
 				media_descriptor_string(bkpinfo->backup_media_type),
 				cd_number_i_want);
 		log_msg(3, tmp);
-		while (what_number_cd_is_this() != cd_number_i_want) {
+		while (what_number_cd_is_this(bkpinfo) != cd_number_i_want) {
 			paranoid_system("sync");
 			if (is_this_device_mounted(MNT_CDROM)) {
 				res =
@@ -1637,12 +1811,14 @@ insist_on_this_cd_number(int cd_number_i_want)
  * @bug No point of `int' return value.
  * @ingroup archiveGroup
  */
-int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
+int interactively_obtain_media_parameters_from_user(struct s_bkpinfo
+													*bkpinfo,
+													bool
+													archiving_to_media)
 // archiving_to_media is TRUE if I'm being called by mondoarchive
 // archiving_to_media is FALSE if I'm being called by mondorestore
 {
 	char *tmp;
-	char *tmp1 = NULL;
 	char *sz_size;
 	char *command;
 	char *comment;
@@ -1659,7 +1835,7 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 	sz_size[0] = '\0';
 	bkpinfo->nonbootable_backup = FALSE;
 
-	// Tape, CD, NFS, ...?
+// Tape, CD, NFS, ...?
 	srandom(getpid());
 	bkpinfo->backup_media_type =
 		(g_restoring_live_from_cd) ? cdr :
@@ -1668,15 +1844,13 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 		log_to_screen("User has chosen not to backup the PC");
 		finish(1);
 	}
-	/* Why asking to remove the media with tape ?
 	if (bkpinfo->backup_media_type == tape && bkpinfo->restore_data) {
-		popup_and_OK("Please remove media from drive(s)");
+		popup_and_OK("Please remove CD/floppy from drive(s)");
 	}
-	*/
 	log_msg(3, "media type = %s",
 			bkptype_to_string(bkpinfo->backup_media_type));
 	if (archiving_to_media) {
-		sensibly_set_tmpdir_and_scratchdir();
+		sensibly_set_tmpdir_and_scratchdir(bkpinfo);
 	}
 	bkpinfo->cdrw_speed = (bkpinfo->backup_media_type == cdstream) ? 2 : 4;
 	bkpinfo->compression_level =
@@ -1685,18 +1859,13 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 		(bkpinfo->backup_media_type == cdstream) ? TRUE : FALSE;
 	mvaddstr_and_log_it(2, 0, " ");
 
-	// Find device's /dev (or SCSI) entry
+// Find device's /dev (or SCSI) entry
 	switch (bkpinfo->backup_media_type) {
 	case cdr:
 	case cdrw:
 	case dvd:
-	case usb:
-		/* Never try to eject a USB device */
-		if (bkpinfo->backup_media_type == usb) {
-			bkpinfo->please_dont_eject = TRUE;
-		}
 		if (archiving_to_media) {
-			if ((bkpinfo->backup_media_type != dvd) && (bkpinfo->backup_media_type != usb)) {
+			if (bkpinfo->backup_media_type != dvd) {
 				if (ask_me_yes_or_no
 					("Is your computer a laptop, or does the CD writer incorporate BurnProof technology?"))
 				{
@@ -1713,7 +1882,7 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 			if (bkpinfo->backup_media_type == dvd) {
 				find_dvd_device(bkpinfo->media_device, FALSE);
 				strcpy(tmp, "1");
-				sprintf(sz_size, "%d", DEFAULT_DVD_DISK_SIZE);	// 4.7 salesman's GB = 4.482 real GB = 4482 MB
+				sprintf(sz_size, "%d", DEFAULT_DVD_DISK_SIZE);	// 4.7 salesman's GB = 4.482 real GB = 4582 MB
 				log_msg(1, "Setting to DVD defaults");
 			} else {
 				strcpy(bkpinfo->media_device, VANILLA_SCSI_CDROM);
@@ -1721,7 +1890,7 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 				strcpy(sz_size, "650");
 				log_msg(1, "Setting to CD defaults");
 			}
-			if ((bkpinfo->backup_media_type != dvd) && (bkpinfo->backup_media_type != usb)) {
+			if (bkpinfo->backup_media_type != dvd) {
 				if (!popup_and_get_string("Speed", comment, tmp, 4)) {
 					log_to_screen("User has chosen not to backup the PC");
 					finish(1);
@@ -1743,13 +1912,12 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 				finish(1);
 			}
 		}
-		/* No break because we continue even for usb */
 	case cdstream:
-		if ((bkpinfo->disaster_recovery) && (bkpinfo->backup_media_type != usb)) {
+		if (bkpinfo->disaster_recovery) {
 			strcpy(bkpinfo->media_device, "/dev/cdrom");
 			log_msg(2, "CD-ROM device assumed to be at %s",
 					bkpinfo->media_device);
-		} else if ((bkpinfo->restore_data && (bkpinfo->backup_media_type != usb))
+		} else if (bkpinfo->restore_data
 				   || bkpinfo->backup_media_type == dvd) {
 			if (!bkpinfo->media_device[0]) {
 				strcpy(bkpinfo->media_device, "/dev/cdrom");
@@ -1762,7 +1930,8 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 						bkpinfo->media_device);
 				sprintf(comment,
 						"Please specify your %s drive's /dev entry",
-						media_descriptor_string(bkpinfo->backup_media_type));
+						media_descriptor_string(bkpinfo->
+												backup_media_type));
 				if (!popup_and_get_string
 					("Device?", comment, bkpinfo->media_device,
 					 MAX_STR_LEN / 4)) {
@@ -1774,43 +1943,30 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 					media_descriptor_string(bkpinfo->backup_media_type),
 					bkpinfo->media_device);
 		} else {
-			if ((find_cdrw_device(bkpinfo->media_device)) && (bkpinfo->backup_media_type != usb)) {
+			if (find_cdrw_device(bkpinfo->media_device)) {
 				bkpinfo->media_device[0] = '\0';
 			}
 			if (bkpinfo->media_device[0]) {
-				if (bkpinfo->backup_media_type == usb) {
-					sprintf(tmp,
-						"I think your %s media corresponds to %s. Is this correct?",
-						media_descriptor_string(bkpinfo->backup_media_type),
-						bkpinfo->media_device);
-				} else {
-					sprintf(tmp,
+				sprintf(tmp,
 						"I think I've found your %s burner at SCSI node %s. Is this correct? (Say no if you have an IDE burner and you are running a 2.6 kernel. You will then be prompted for further details.)",
-						media_descriptor_string(bkpinfo->backup_media_type),
+						media_descriptor_string(bkpinfo->
+												backup_media_type),
 						bkpinfo->media_device);
-				}
 				if (!ask_me_yes_or_no(tmp)) {
 					bkpinfo->media_device[0] = '\0';
 				}
 			}
 			if (!bkpinfo->media_device[0]) {
-				if (bkpinfo->backup_media_type == usb) {
-					i = popup_and_get_string("/dev entry?",
-										 "What is the /dev entry of your USB Disk/Key, please?",
-										 bkpinfo->media_device,
-										 MAX_STR_LEN / 4);
-				} else {
-					if (g_kernel_version < 2.6) {
-						i = popup_and_get_string("Device node?",
+				if (g_kernel_version < 2.6) {
+					i = popup_and_get_string("Device node?",
 											 "What is the SCSI node of your CD (re)writer, please?",
 											 bkpinfo->media_device,
 											 MAX_STR_LEN / 4);
-					} else {
-						i = popup_and_get_string("/dev entry?",
+				} else {
+					i = popup_and_get_string("/dev entry?",
 											 "What is the /dev entry of your CD (re)writer, please?",
 											 bkpinfo->media_device,
 											 MAX_STR_LEN / 4);
-					}
 				}
 				if (!i) {
 					log_to_screen("User has chosen not to backup the PC");
@@ -1832,7 +1988,7 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 		}
 	case tape:
 
-		if ((!bkpinfo->restore_mode) && (find_tape_device_and_size(bkpinfo->media_device, sz_size))) {
+		if (find_tape_device_and_size(bkpinfo->media_device, sz_size)) {
 			log_msg(3, "Ok, using vanilla scsi tape.");
 			strcpy(bkpinfo->media_device, VANILLA_SCSI_TAPE);
 			if ((fin = fopen(bkpinfo->media_device, "r"))) {
@@ -1885,8 +2041,6 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 	      { log_to_screen("User has chosen not to backup the PC"); finish(1); }
 	  }
 */
-		bkpinfo->use_obdr = ask_me_yes_or_no
-			("Do you want to activate OBDR support for your tapes ?");
 		if (sz_size[0] == '\0') {
 			bkpinfo->media_size[0] = 0;
 		} else {
@@ -1912,10 +2066,6 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 
 
 	case nfs:
-		/* Never try to eject a NFS device */
-		bkpinfo->please_dont_eject = TRUE;
-
-		/* Initiate bkpinfo nfs_mount path from running environment if not already done */
 		if (!bkpinfo->nfs_mount[0]) {
 			strcpy(bkpinfo->nfs_mount,
 				   call_program_and_get_last_line_of_output
@@ -1951,27 +2101,22 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 			strcpy(bkpinfo->isodir,
 				   call_program_and_get_last_line_of_output(command));
 
-			if (!bkpinfo->restore_data) {
-				sprintf(comment,
+			sprintf(comment,
 					"How much data (in Megabytes) will each media store?");
-				if (!popup_and_get_string("Size", comment, sz_size, 5)) {
-					log_to_screen("User has chosen not to backup the PC");
-					finish(1);
-				}
-			} else {
-				strcpy(sz_size, "0");
+			if (!popup_and_get_string("Size", comment, sz_size, 5)) {
+				log_to_screen("User has chosen not to backup the PC");
+				finish(1);
 			}
 			for (i = 0; i <= MAX_NOOF_MEDIA; i++) {
 				bkpinfo->media_size[i] = atoi(sz_size);
 			}
-			if (bkpinfo->media_size[0] < 0) {
+			if (bkpinfo->media_size[0] <= 0) {
 				log_to_screen("User has chosen not to backup the PC");
 				finish(1);
 			}
 		}
 		if (bkpinfo->disaster_recovery) {
-			sprintf(command ,"umount %s/isodir 2> /dev/null", bkpinfo->tmpdir);
-			system(command);
+			system("umount /tmp/isodir 2> /dev/null");
 			if (!popup_and_get_string
 				("NFS share", "Which remote NFS share should I mount?",
 				 bkpinfo->nfs_mount, MAX_STR_LEN)) {
@@ -1979,18 +2124,14 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 				finish(1);
 			}
 		}
-		/* Initiate bkpinfo isodir path from running environment if mount already done */
-		if (is_this_device_mounted(bkpinfo->nfs_mount)) {
-			strcpy(bkpinfo->isodir,
-				   call_program_and_get_last_line_of_output
-				   ("mount | grep \":\" | cut -d' ' -f3 | head -n1"));
-		} else {
-			sprintf(bkpinfo->isodir, "%s/nfsdir", bkpinfo->tmpdir);
+		if (!is_this_device_mounted(bkpinfo->nfs_mount)) {
+			sprintf(bkpinfo->isodir, "/tmp/isodir.mondo.%d",
+					(int) (random() % 32768));
 			sprintf(command, "mkdir -p %s", bkpinfo->isodir);
 			run_program_and_log_output(command, 5);
 			sprintf(tmp, "mount -t nfs -o nolock %s %s", bkpinfo->nfs_mount,
 					bkpinfo->isodir);
-			run_program_and_log_output(tmp, 3);
+			run_program_and_log_output(tmp, 5);
 			malloc_string(g_selfmounted_isodir);
 			strcpy(g_selfmounted_isodir, bkpinfo->isodir);
 		}
@@ -2009,8 +2150,8 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 		strcpy(bkpinfo->nfs_remote_dir, tmp);
 		// check whether writable - we better remove surrounding spaces for this
 		strip_spaces(bkpinfo->nfs_remote_dir);
-		asprintf(&tmp1,"%s/%s/.dummy.txt", bkpinfo->isodir,bkpinfo->nfs_remote_dir);
-		sprintf(command, "echo hi > %s", tmp1);
+		sprintf(command, "echo hi > %s/%s/.dummy.txt", bkpinfo->isodir,
+				bkpinfo->nfs_remote_dir);
 		while (run_program_and_log_output(command, FALSE)) {
 			strcpy(tmp, bkpinfo->nfs_remote_dir);
 			sprintf(prompt,
@@ -2024,13 +2165,9 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 			strcpy(bkpinfo->nfs_remote_dir, tmp);
 			// check whether writable - we better remove surrounding space s for this
 			strip_spaces(bkpinfo->nfs_remote_dir);
-			paranoid_free(tmp1);
-			asprintf(&tmp1,"%s/%s/.dummy.txt", bkpinfo->isodir,bkpinfo->nfs_remote_dir);
-			sprintf(command, "echo hi > %s", tmp1);
+			sprintf(command, "echo hi > %s/%s/.dummy.txt", bkpinfo->isodir,
+					bkpinfo->nfs_remote_dir);
 		}
-		unlink(tmp1);
-		paranoid_free(tmp1);
-
 		if (!popup_and_get_string
 			("Prefix.",
 			 "Please enter the prefix that will be prepended to your ISO filename.  Example: machine1 to obtain machine1-[1-9]*.iso files",
@@ -2052,7 +2189,7 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 		if (!bkpinfo->disaster_recovery) {
 			if (!popup_and_get_string
 				("Storage dir.",
-				 "Please enter the full path name to the directory for your ISO images.  Example: /mnt/raid0_0",
+				 "Please enter the full path that contains your ISO images.  Example: /mnt/raid0_0",
 				 bkpinfo->isodir, MAX_STR_LEN / 4)) {
 				log_to_screen("User has chosen not to backup the PC");
 				finish(1);
@@ -2187,16 +2324,11 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 		if (!popup_and_get_string
 			("Exclude paths",
 			 "Please enter paths which you do NOT want to backup. Separate them with spaces. NB: /tmp and /proc are always excluded. :-) Just hit 'Enter' if you want to do a full system backup.",
-			 bkpinfo->exclude_paths, (4*MAX_STR_LEN)-1)) {
+			 bkpinfo->exclude_paths, MAX_STR_LEN)) {
 			log_to_screen("User has chosen not to backup the PC");
 			finish(1);
 		}
-// Interactive mode:
-#ifdef __IA64__
-		bkpinfo->make_cd_use_lilo = TRUE;
-#else
 		bkpinfo->make_cd_use_lilo = FALSE;
-#endif
 		bkpinfo->backup_data = TRUE;
 		bkpinfo->verify_data =
 			ask_me_yes_or_no
@@ -2247,7 +2379,6 @@ int interactively_obtain_media_parameters_from_user(bool archiving_to_media)
 	log_it("exclude_paths = '%s'", bkpinfo->exclude_paths);
 	log_it("scratchdir = '%s'", bkpinfo->scratchdir);
 	log_it("tmpdir = '%s'", bkpinfo->tmpdir);
-	log_it("image_devs = '%s'", bkpinfo->image_devs);
 	log_it("boot_device = '%s' (loader=%c)", bkpinfo->boot_device,
 		   bkpinfo->boot_loader);
 	if (bkpinfo->media_size[0] < 0) {
@@ -2288,10 +2419,10 @@ char *list_of_NFS_devices_and_mounts(void)
 	malloc_string(exclude_these_directories);
 	strcpy(exclude_these_directories,
 		   call_program_and_get_last_line_of_output
-		   ("mount -t coda,ncpfs,nfs,smbfs,cifs,afs,gfs,ocfs,ocfs2,mvfs,nsspool,nssvol | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("mount -t coda,ncpfs,nfs,smbfs,cifs,afs,mvfs | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	strcpy(exclude_these_devices,
 		   call_program_and_get_last_line_of_output
-		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs|afs|gfs|ocfs|ocfs2|mvfs|nsspool|nsvol) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs|mvfs) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	sprintf(result_sz, "%s %s", exclude_these_directories,
 			exclude_these_devices);
 	paranoid_free(exclude_these_devices);
@@ -2318,10 +2449,10 @@ char *list_of_NFS_mounts_only(void)
 	malloc_string(exclude_these_directories);
 	strcpy(exclude_these_directories,
 		   call_program_and_get_last_line_of_output
-		   ("mount -t coda,ncpfs,nfs,smbfs,cifs,afs,gfs,ocfs,ocfs2,mvfs,nsspool,nssvol | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("mount -t coda,ncpfs,nfs,smbfs,cifs,afs,mvfs | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	strcpy(exclude_these_devices,
 		   call_program_and_get_last_line_of_output
-		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs|afs|gfs|ocfs|ocfs2|mvfs|nsspool|nssvol) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs|mvfs) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	sprintf(result_sz, "%s", exclude_these_directories);
 	paranoid_free(exclude_these_devices);
 	paranoid_free(exclude_these_directories);
@@ -2368,7 +2499,7 @@ void make_fifo(char *store_name_here, char *stub)
  * @param bkpinfo The backup information structure. @c bkpinfo->tmpdir and @c bkpinfo->scratchdir will be set.
  * @ingroup utilityGroup
  */
-void sensibly_set_tmpdir_and_scratchdir()
+void sensibly_set_tmpdir_and_scratchdir(struct s_bkpinfo *bkpinfo)
 {
 	char *tmp, *command, *sz;
 
@@ -2380,11 +2511,11 @@ void sensibly_set_tmpdir_and_scratchdir()
 #ifdef __FreeBSD__
 	strcpy(tmp,
 		   call_program_and_get_last_line_of_output
-		   ("LANGUAGE=C df -m -P -t nonfs,msdosfs,ntfs,smbfs,smb,cifs,afs,gfs,ocfs,ocfs2,mvfs,nsspool,nssvol | tr -s '\t' ' ' | grep -vE \"none|Filesystem\" | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
+		   ("df -m -P -t nonfs,msdosfs,ntfs,smbfs,smb,cifs,afs,mvfs | tr -s '\t' ' ' | grep -vE \"none|Filesystem\" | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
 #else
 	strcpy(tmp,
 		   call_program_and_get_last_line_of_output
-		   ("LANGUAGE=C df -m -P -x nfs -x vfat -x ntfs -x smbfs -x smb -x cifs -x afs -x gfs -x ocfs -x ocfs2 -x mvfs -x nsspool -x nssvol| sed 's/                  /devdev/' | tr -s '\t' ' ' | grep -vE \"none|Filesystem|/dev/shm\" | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
+		   ("df -m -P -x nfs -x vfat -x ntfs -x smbfs -x smb -x cifs -x afs -x mvfs | sed 's/                  /devdev/' | tr -s '\t' ' ' | grep -vE \"none|Filesystem|/dev/shm\" | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
 #endif
 
 	if (tmp[0] != '/') {
@@ -2395,13 +2526,17 @@ void sensibly_set_tmpdir_and_scratchdir()
 	if (!tmp[0]) {
 		fatal_error("I couldn't figure out the tempdir!");
 	}
-	setup_tmpdir(tmp);
+	sprintf(bkpinfo->tmpdir, "%s/tmp.mondo.%d", tmp,
+			(int) (random() % 32768));
 	log_it("bkpinfo->tmpdir is being set to %s", bkpinfo->tmpdir);
 
 	sprintf(bkpinfo->scratchdir, "%s/mondo.scratch.%d", tmp,
 			(int) (random() % 32768));
 	log_it("bkpinfo->scratchdir is being set to %s", bkpinfo->scratchdir);
 
+	sprintf(g_erase_tmpdir_and_scratchdir, "rm -Rf %s %s", bkpinfo->tmpdir,
+			bkpinfo->scratchdir);
+
 	sprintf(command, "rm -Rf %s/tmp.mondo.* %s/mondo.scratch.*", tmp, tmp);
 	paranoid_system(command);
 	paranoid_free(tmp);
@@ -2466,7 +2601,7 @@ bool set_dev_to_this_if_rx_OK(char *output, char *dev)
  * @note If the CD is not mounted, it will be mounted
  * (and remain mounted after this function returns).
  */
-int what_number_cd_is_this()
+int what_number_cd_is_this(struct s_bkpinfo *bkpinfo)
 {
 	int cd_number = -1;
 	char *mountdev;
@@ -2497,11 +2632,7 @@ int what_number_cd_is_this()
 		find_cdrom_device(bkpinfo->media_device, FALSE);
 	}
 	if (!is_this_device_mounted(MNT_CDROM)) {
-		if (bkpinfo->backup_media_type == usb) {
-			mount_USB_here(mountdev, MNT_CDROM);
-		} else {
-			mount_CDROM_here(mountdev, MNT_CDROM);
-		}
+		mount_CDROM_here(mountdev, MNT_CDROM);
 	}
 	cd_number =
 		atoi(last_line_of_file(MNT_CDROM "/archives/THIS-CD-NUMBER"));
@@ -2658,6 +2789,8 @@ char which_boot_loader(char *which_device)
 		log_OS_error("Cannot pclose pdrives");
 	}
 	log_it("%d grubs and %d lilos\n", count_grubs, count_lilos);
+	paranoid_free(list_drives_cmd);
+	paranoid_free(current_drive);
 	if (count_grubs && !count_lilos) {
 		return ('G');
 	} else if (count_lilos && !count_grubs) {
@@ -2666,49 +2799,8 @@ char which_boot_loader(char *which_device)
 		log_it("I'll bet you used to use LILO but switched to GRUB...");
 		return ('G');
 	} else {
-		// We need to look on each partition then
-		sprintf(list_drives_cmd,
-			"parted2fdisk -l 2>/dev/null | grep -E \"^/dev/\" | tr -s ':' ' ' | tr -s ' ' '\n' | grep /dev/");
-		log_it("list_drives_cmd = %s", list_drives_cmd);
-
-		if (!(pdrives = popen(list_drives_cmd, "r"))) {
-			log_OS_error("Unable to open list of drives");
-			paranoid_free(list_drives_cmd);
-			paranoid_free(current_drive);
-			return ('\0');
-		}
-		for (fgets(current_drive, MAX_STR_LEN, pdrives); !feof(pdrives);
-		 	fgets(current_drive, MAX_STR_LEN, pdrives)) {
-			strip_spaces(current_drive);
-			log_it("looking at partition %s's BR", current_drive);
-			if (does_string_exist_in_boot_block(current_drive, "GRUB")) {
-				count_grubs++;
-				strcpy(which_device, current_drive);
-				break;
-			}
-			if (does_string_exist_in_boot_block(current_drive, "LILO")) {
-				count_lilos++;
-				strcpy(which_device, current_drive);
-				break;
-			}
-		}
-		if (pclose(pdrives)) {
-			log_OS_error("Cannot pclose pdrives");
-		}
-		log_it("%d grubs and %d lilos\n", count_grubs, count_lilos);
-		paranoid_free(list_drives_cmd);
-		paranoid_free(current_drive);
-		if (count_grubs && !count_lilos) {
-			return ('G');
-		} else if (count_lilos && !count_grubs) {
-			return ('L');
-		} else if (count_grubs == 1 && count_lilos == 1) {
-			log_it("I'll bet you used to use LILO but switched to GRUB...");
-			return ('G');
-		} else {
-			log_it("Unknown boot loader");
-			return ('U');
-		}
+		log_it("Unknown boot loader");
+		return ('U');
 	}
 }
 #endif
