@@ -1,5 +1,5 @@
 /* libmondo-mountlist.c                            subroutines for handling mountlist
-   $Id: libmondo-mountlist.c 1909 2008-04-14 11:18:58Z bruno $
+   $Id: libmondo-mountlist.c 1303 2007-04-15 00:33:22Z bruno $
 
 
 
@@ -60,10 +60,13 @@
 #include "newt-specific-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-mountlist.c 1909 2008-04-14 11:18:58Z bruno $";
+//static char cvsid[] = "$Id: libmondo-mountlist.c 1303 2007-04-15 00:33:22Z bruno $";
 
-/* Reference to global bkpinfo */
-extern struct s_bkpinfo *bkpinfo;
+/**
+ * A global copy of @c bkpinfo, to aid in debugging. As the name implies, <em>don't use this</em>.
+ * @ingroup globalGroup
+ */
+struct s_bkpinfo *g_bkpinfo_DONTUSETHIS = NULL;
 
 /**
  * @addtogroup mountlistGroup
@@ -367,6 +370,8 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 				physical_drive_size - amount_allocated, drive);
 		log_it(tmp);
 		strcat(flaws_str, tmp);
+		/* BERLIOS: Flawed since rev 1 !! */
+		res++;
 	}
 	if (res) {
 		return (FALSE);
@@ -515,6 +520,8 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 				physical_drive_size - amount_allocated, drive);
 		log_it(tmp);
 		strcat(flaws_str, tmp);
+		/* BERLIOS: Flawed since rev 1 !! */
+		res++;
 	}
 
   endoffunc:
@@ -951,11 +958,11 @@ int load_mountlist(struct mountlist_itself *mountlist, char *fname)
 	while (!feof(fin)) {
 #if linux
 		sscanf(incoming,
-			   "%s %s %s %s %s",
+			   "%s %s %s %s %s %s",
 			   mountlist->el[items].device,
 			   mountlist->el[items].mountpoint,
 			   mountlist->el[items].format,
-			   siz, mountlist->el[items].label);
+			   siz, mountlist->el[items].label, mountlist->el[items].uuid);
 #elif __FreeBSD__
 		sscanf(incoming,
 			   "%s %s %s %s",
@@ -963,6 +970,7 @@ int load_mountlist(struct mountlist_itself *mountlist, char *fname)
 			   mountlist->el[items].mountpoint,
 			   mountlist->el[items].format, siz);
 		strcpy(mountlist->el[items].label, "");
+		strcpy(mountlist->el[items].uuid, "");
 #endif
 
 		if (!strcmp(mountlist->el[items].device, "/proc") ||
@@ -1011,12 +1019,13 @@ int load_mountlist(struct mountlist_itself *mountlist, char *fname)
 			}
 
 			sprintf(tmp,
-					"%s %s %s %lld %s",
+					"%s %s %s %lld %s %s",
 					mountlist->el[items].device,
 					mountlist->el[items].mountpoint,
 					mountlist->el[items].format,
 					mountlist->el[items].size,
-					mountlist->el[items].label);
+					mountlist->el[items].label,
+					mountlist->el[items].uuid);
 
 			log_it(tmp);
 			items++;
@@ -1062,7 +1071,7 @@ int save_mountlist_to_disk(struct mountlist_itself *mountlist, char *fname)
 				"%-15s %-15s %-15s %-15lld %-15s\n",
 				mountlist->el[i].device, mountlist->el[i].mountpoint,
 				mountlist->el[i].format, mountlist->el[i].size,
-				mountlist->el[i].label);
+				mountlist->el[i].label, mountlist->el[i].uuid);
 	}
 	paranoid_fclose(fout);
 	return (0);
