@@ -1,8 +1,100 @@
 /* newt-specific.c
-   $Id: newt-specific.c 2030 2008-10-02 23:50:44Z bruno $
+   $Id: newt-specific.c 1316 2007-04-16 14:50:28Z bruno $
 
   subroutines which do display-type things
   and use the newt library to do them
+.
+
+10/02
+- tell user not to bother mailing list unless snapshot
+
+07/09
+- finish() calls g_erase_tmpdir_and_scratchdir to erase
+  tmpdir and scratchdir at end
+
+06/25
+- tried to fix newtFinished() segfault
+
+04/13/2004
+- lots of char[]'s are now *char's
+
+10/09/2003
+- allow DVD option for text-mode which_backup_type() too
+
+09/28
+- log_to_screen() now takes printf-like parameters
+
+09/26
+- b5 now means dvd (was udev)
+
+09/16
+- echo fatal error to screen before exiting
+- working on fatal_error()
+- swapped g_blurb2 and g_blurb3 in a few places to fix a GUI problem;
+  it shouldn't have worked! :) I think there's a mem-leak somewhere
+
+09/13
+- changed some '\n' occurrences to '\r\n'
+
+09/09
+- fixed silly bug in log_to_screen()
+
+07/02
+- modified popup_and_get_string()
+
+05/03
+- cleaned up fatal_error()
+
+04/30
+- fixed popup_changelist_*()
+
+04/27
+- replace newtFinished() and newtInit() with
+  newtSuspend() and newtResume()
+
+04/25
+- after comparing, popup the changelist for the user
+- moved the relevant code from compare_to_cds() into
+  compare_mode(), so that tape users may benefit too
+
+04/24
+- added lots of assert()'s and log_OS_error()'s
+- cleaned up a few uninitialized strings (valgrind)
+
+04/22
+- line 1181 - 'read from' (not 'restoring from')
+
+03/15/2003
+- fixed potentially infinite loop in log_to_screen (Tom Mortell)
+
+12/04/2002
+- changed "Pick file" to "Non-matching files"
+
+11/28
+- finish() now unregisters Mondo's pid
+
+10/28
+- unmount tmpfs if fatal_error
+
+10/04
+- more XMondo-related work
+- added some =NULL's in a few places
+
+09/01 - 09/30
+- write log_to_screen()'s string to stdout if in text mode
+- update_progress_form_FULL(), update_evalcall_form(),
+  popup_and_*(), ask_me_yes_or_no() now write
+  XMondo-friendly info to stdout if in text mode
+- if fatal error then popup and say what the fatal error is
+- run_program_and_log_output() now takes boolean operator to specify
+  whether it will log its activities in the event of _success_
+- added popup_changelist_from_file(char*source_file, char*topic, void*call_if_entry_selected(void*));
+
+08/07
+- added some functions to let user choose backup dev + format
+
+07/27
+- created
 */
 
 
@@ -30,16 +122,13 @@ extern "C" {
 
 extern char *MONDO_LOGFILE;
 
-extern struct s_bkpinfo *bkpinfo;
-
 /*@unused@*/
-//static char cvsid[] = "$Id: newt-specific.c 2030 2008-10-02 23:50:44Z bruno $";
+//static char cvsid[] = "$Id: newt-specific.c 1316 2007-04-16 14:50:28Z bruno $";
 
 	extern pid_t g_mastermind_pid;
-	/*
 	extern char *g_tmpfs_mountpt;
-	*/
 	extern char *g_boot_mountpt;
+	extern char *g_mondo_home;
 	extern char *ps_options;
 	extern char *ps_proc_id;
 
@@ -99,6 +188,7 @@ extern struct s_bkpinfo *bkpinfo;
 	char g_xmondo_stdin[MAX_NEWT_COMMENT_LEN],	///< ... @bug Unneeded w/current XMondo.
 	 g_xmondo_stdout[MAX_NEWT_COMMENT_LEN];	///< .... @bug Unneeded w/current XMondo.
 	bool g_called_by_xmondo = FALSE;	///< @bug Unneeded w/current XMondo.
+	char *g_erase_tmpdir_and_scratchdir;	///< The command to run to erase the tmpdir and scratchdir at the end of Mondo.
 	char *g_selfmounted_isodir;	///< Holds the NFS mountpoint if mounted via mondoarchive.
 
 /* @} - end of globalGroup */
@@ -130,7 +220,7 @@ extern struct s_bkpinfo *bkpinfo;
 			while (1) {
 				system("sync");
 				printf
-					("---promptdialogYN---1--- %s\n---promptdialogYN---Q--- [yes] [no] ---\n--> ",
+					("---promptdialogYN---1--- %s\r\n---promptdialogYN---Q--- [yes] [no] ---\r\n--> ",
 					 prompt);
 				(void) fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
 				if (tmp[strlen(tmp) - 1] == '\n')
@@ -176,7 +266,7 @@ extern struct s_bkpinfo *bkpinfo;
 		if (g_text_mode) {
 			system("sync");
 			printf
-				("---promptdialogOKC---1--- %s\n---promptdialogOKC---Q--- [OK] [Cancel] ---\n--> ",
+				("---promptdialogOKC---1--- %s\r\n---promptdialogOKC---Q--- [OK] [Cancel] ---\r\n--> ",
 				 prompt);
 			(void) fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
 			if (tmp[strlen(tmp) - 1] == '\n')
@@ -259,15 +349,9 @@ extern struct s_bkpinfo *bkpinfo;
 void kill_anything_like_this(char *str) {
 
 char *tmp = NULL;
-char *tmp1 = NULL;
 
-	asprintf(&tmp,"ps %s | grep \" %s \" | awk '{print %s;}' | grep -v \"grep\"", ps_options, str , ps_proc_id);
+	asprintf(&tmp,"kill `ps %s | grep \" %s \" | awk '{print %s;}' | grep -v \"grep\"`", ps_options, str , ps_proc_id);
 	run_program_and_log_output(tmp, TRUE);
-	if (strlen(tmp) > 0) {
-		asprintf(&tmp1,"kill `%s`", tmp);
-		run_program_and_log_output(tmp1, TRUE);
-		paranoid_free(tmp1);
-	}
 	paranoid_free(tmp);
 }
 
@@ -325,16 +409,18 @@ char *tmp1 = NULL;
 		kill_anything_like_this("tmp.mondo");
 		kill_anything_like_this("ntfsclone");
 		sync();
-		/*
 		sprintf(tmp, "umount %s", g_tmpfs_mountpt);
+		chdir("/");
 		for (i = 0; i < 10 && run_program_and_log_output(tmp, 5); i++) {
 			log_msg(2, "Waiting for child processes to terminate");
 			sleep(1);
 			run_program_and_log_output(tmp, 5);
 		}
-		*/
 
-		chdir("/");
+		if (g_erase_tmpdir_and_scratchdir[0]) {
+			run_program_and_log_output(g_erase_tmpdir_and_scratchdir, 5);
+		}
+
 		if (g_selfmounted_isodir) {
 			sprintf(command, "umount %s", g_selfmounted_isodir);
 			run_program_and_log_output(command, 5);
@@ -350,7 +436,7 @@ char *tmp1 = NULL;
 		}
 
 		printf("---FATALERROR--- %s\n", error_string);
-		sprintf(command, "gzip -9c %s > %s/MA.log.gz 2> /dev/null", MONDO_LOGFILE, MONDO_CACHE);
+		sprintf(command, "gzip -9c %s > /tmp/MA.log.gz 2> /dev/null", MONDO_LOGFILE);
 		system(command);
 		printf
 				("If you require technical support, please contact the mailing list.\n");
@@ -358,8 +444,9 @@ char *tmp1 = NULL;
 		printf
 				("The list's members can help you, if you attach that file to your e-mail.\n");
 		printf("Log file: %s\n", MONDO_LOGFILE);
-		if (does_file_exist(MONDO_CACHE"/MA.log.gz")) {
-			printf("FYI, I have gzipped the log and saved it to "MONDO_CACHE"/MA.log.gz.\n");
+		if (does_file_exist("/tmp/MA.log.gz")) {
+			printf
+				("FYI, I have gzipped the log and saved it to /tmp/MA.log.gz\n");
 		}
 		printf("Mondo has aborted.\n");
 		register_pid(0, "mondo");	// finish() does this too, FYI
@@ -381,7 +468,8 @@ char *tmp1 = NULL;
  */
 	void
 	 finish(int signal) {
-		char *command = NULL;
+		char *command;
+		malloc_string(command);
 
 		/*  if (signal==0) { popup_and_OK("Please press <enter> to quit."); } */
 
@@ -390,14 +478,20 @@ char *tmp1 = NULL;
 		register_pid(0, "mondo");
 		chdir("/");
 		run_program_and_log_output("umount " MNT_CDROM, FALSE);
-		if (g_selfmounted_isodir) {
-			asprintf(&command, "umount %s", g_selfmounted_isodir);
-			run_program_and_log_output(command, 1);
-			paranoid_free(command);
-			asprintf(&command, "rmdir %s", g_selfmounted_isodir);
-			run_program_and_log_output(command, 1);
-			paranoid_free(command);
+		run_program_and_log_output("rm -Rf /mondo.scratch.* /tmp.mondo.*",
+								   FALSE);
+		if (g_erase_tmpdir_and_scratchdir) {
+			run_program_and_log_output(g_erase_tmpdir_and_scratchdir, 1);
 		}
+		if (g_selfmounted_isodir) {
+			sprintf(command, "umount %s", g_selfmounted_isodir);
+			run_program_and_log_output(command, 1);
+			sprintf(command, "rmdir %s", g_selfmounted_isodir);
+			run_program_and_log_output(command, 1);
+		}
+//  iamhere("foo");
+		/* system("clear"); */
+//  iamhere("About to call newtFinished");
 		if (!g_text_mode) {
 			if (does_file_exist("/THIS-IS-A-RAMDISK")) {
 				log_msg(1, "Calling newtFinished()");
@@ -407,14 +501,10 @@ char *tmp1 = NULL;
 				newtSuspend();
 			}
 		}
+//  system("clear");
+//  iamhere("Finished calling newtFinished");
 		printf("Execution run ended; result=%d\n", signal);
 		printf("Type 'less %s' to see the output log\n", MONDO_LOGFILE);
-		if (strstr(bkpinfo->tmpdir ,"mondo.tmp.") != NULL) {
-			asprintf(&command, "rm -Rf %s %s", bkpinfo->tmpdir, bkpinfo->scratchdir);
-			system(command);
-			paranoid_free(command);
-		}
-		paranoid_free(bkpinfo);
 		free_libmondo_global_strings();
 		exit(signal);
 	}
@@ -704,7 +794,7 @@ char *tmp1 = NULL;
 		log_msg(0, prompt);
 		if (g_text_mode) {
 			printf
-				("---promptpopup---1--- %s\n---promptpopup---Q--- [OK] ---\n--> ",
+				("---promptpopup---1--- %s\r\n---promptpopup---Q--- [OK] ---\r\n--> ",
 				 prompt);
 			while (((ch = getchar()) != '\n') && (ch != EOF));
 		} else {
@@ -746,7 +836,7 @@ char *tmp1 = NULL;
 
 		if (g_text_mode) {
 			printf
-				("---promptstring---1--- %s\n---promptstring---2--- %s\n---promptstring---Q---\n-->  ",
+				("---promptstring---1--- %s\r\n---promptstring---2--- %s\r\n---promptstring---Q---\r\n-->  ",
 				 title, b);
 			(void) fgets(output, maxsize, stdin);
 			if (output[strlen(output) - 1] == '\n')
@@ -994,9 +1084,7 @@ char *tmp1 = NULL;
 			time_remaining = 0;
 		}
 		if (!g_text_mode) {
-			if (g_isoform_header) {
-				newtLabelSetText(g_isoform_header, g_isoform_header_str);
-			}
+			newtLabelSetText(g_isoform_header, g_isoform_header_str);
 		}
 		g_mysterious_dot_counter = (g_mysterious_dot_counter + 1) % 27;
 		if ((percentage < 3 && g_isoform_old_progress < 3)
@@ -1032,18 +1120,16 @@ char *tmp1 = NULL;
 					sprintf(taskprogress + strlen(taskprogress),
 							"] %3d%% done; %2ld:%02ld to go", percentage,
 							time_remaining / 60, time_remaining % 60);
-					printf("---evalcall---1--- %s\n",
+					printf("---evalcall---1--- %s\r\n",
 						   g_isoform_header_str);
-					printf("---evalcall---2--- %s\n", taskprogress);
-					printf("---evalcall---E---\n");
+					printf("---evalcall---2--- %s\r\n", taskprogress);
+					printf("---evalcall---E---\r\n");
 				}
 			} else {
 				newtScaleSet(g_isoform_scale,
 							 (unsigned long long) percentage);
-				if (g_isoform_pcline) {
-					newtLabelSetText(g_isoform_pcline, pcline_str);
-				}
-				if ((percentage >= 3) && (g_isoform_timeline)) {
+				newtLabelSetText(g_isoform_pcline, pcline_str);
+				if (percentage >= 3) {
 					newtLabelSetText(g_isoform_timeline, timeline_str);
 				}
 			}
@@ -1165,9 +1251,9 @@ char *tmp1 = NULL;
 				percentage, 100 - percentage);
 
 		if (g_text_mode) {
-			printf("---progress-form---1--- %s\n", blurb1);
-			printf("---progress-form---2--- %s\n", blurb2);
-			printf("---progress-form---3--- %s\n", blurb3);
+			printf("---progress-form---1--- %s\r\n", blurb1);
+			printf("---progress-form---2--- %s\r\n", blurb2);
+			printf("---progress-form---3--- %s\r\n", blurb3);
 			printf("---progress-form---E---\n");
 			sprintf(taskprogress, "TASK:  [");
 			for (i = 0; i < percentage; i += 5) {
@@ -1184,27 +1270,19 @@ char *tmp1 = NULL;
 			sprintf(taskprogress + strlen(taskprogress),
 					" done; %2ld:%02ld to go",
 					time_remaining / 60, time_remaining % 60);
-			printf("---progress-form---4--- %s\n", taskprogress);
+			printf("---progress-form---4--- %s\r\n", taskprogress);
 		} else {
 			center_string(blurb1, 54);
 			center_string(blurb2, 54);
 			center_string(blurb3, 54);
-			if (g_blurb1) {
-				newtLabelSetText(g_blurb1, blurb1);
-			}
-			if (g_blurb2) {
-				newtLabelSetText(g_blurb2, blurb3);
-			}
-			if (g_blurb3) {
-				newtLabelSetText(g_blurb3, blurb2);
-			}
+			newtLabelSetText(g_blurb1, blurb1);
+			newtLabelSetText(g_blurb2, blurb3);
+			newtLabelSetText(g_blurb3, blurb2);
 			newtScaleSet(g_scale, (unsigned long long) g_current_progress);
-			if ((percentage >= 2) && (g_timeline)) {
+			if (percentage >= 2) {
 				newtLabelSetText(g_timeline, timeline_str);
 			}
-			if (g_percentline) {
-				newtLabelSetText(g_percentline, percentline_str);
-			}
+			newtLabelSetText(g_percentline, percentline_str);
 			newtRefresh();
 		}
 		paranoid_free(percentline_str);
@@ -1303,7 +1381,7 @@ char *tmp1 = NULL;
 		b1 = newtButton(1, 1, "CD-R disks ");
 		b2 = newtButton(17, 1, "CD-RW disks");
 		b3 = newtButton(1, 9, "Tape drive ");
-		b4 = newtButton(17, 5, "USB Key/Disk");
+		b4 = newtButton(17, 5, "CD streamer");
 		b5 = newtButton(1, 5, " DVD disks ");
 		b6 = newtButton(17, 9, " NFS mount ");
 		b7 = newtButton(1, 13, " Hard disk ");
@@ -1321,7 +1399,7 @@ char *tmp1 = NULL;
 		} else if (b_res == b3) {
 			output = tape;
 		} else if (b_res == b4) {
-			output = usb;
+			output = cdstream;
 		} else if (b_res == b5) {
 			output = dvd;
 		} else if (b_res == b6) {
@@ -1370,7 +1448,7 @@ char *tmp1 = NULL;
 		newtCenteredWindow(34, 13, "How much compression?");
 		b1 = newtButton(4, 1, "Maximum");
 		b2 = newtButton(18, 1, "Average");
-		b3 = newtButton(4, 5, "Minimum");
+		b3 = newtButton(4, 5, "Minumum");
 		b4 = newtButton(18, 5, " None  ");
 		b5 = newtButton(4, 9, "         Exit        ");
 		myForm = newtForm(NULL, NULL, 0);
@@ -1406,13 +1484,12 @@ char *tmp1 = NULL;
 	int load_filelist_into_array(struct s_filelist *filelist,
 								 char *source_file) {
 		int i;
-		int j = 0;
 		bool done;
 		char *tmp;
-		char *tmp1 = NULL;
-		FILE *fin, *fout;
+		FILE *fin;
 		struct s_filelist_entry dummy_fle;
 
+		malloc_string(tmp);
 		assert(filelist != NULL);
 		assert_string_is_neither_NULL_nor_zerolength(source_file);
 
@@ -1421,16 +1498,9 @@ char *tmp1 = NULL;
 			log_OS_error(source_file);
 			log_msg(2, "Can't open %s; therefore, cannot popup list",
 					source_file);
+			paranoid_free(tmp);
 			return (1);
 		}
-
-		asprintf(&tmp1,"%s/icantfindthesefiles.txt",bkpinfo->tmpdir);
-		if (!(fout = fopen(tmp1, "a"))) {
-			log_msg(2, "Can't write to %s", tmp1);
-			return(1);
-			}
-
-		malloc_string(tmp);
 		log_msg(2, "Loading %s", source_file);
 		for (filelist->entries = 0; filelist->entries <= ARBITRARY_MAXIMUM;
 			 filelist->entries++) {
@@ -1449,10 +1519,7 @@ char *tmp1 = NULL;
 			if (i < 2) {
 				goto god_i_hate_gotos;
 			}
-			if (!does_file_exist(tmp) && !feof(fin)) {
-				j++;
-				fprintf(fout, "%s\n", tmp);
-
+			if (!does_file_exist(tmp)) {
 				goto god_i_hate_gotos;
 			}
 			filelist->el[filelist->entries].severity =
@@ -1463,11 +1530,6 @@ char *tmp1 = NULL;
 			}
 		}
 		paranoid_fclose(fin);
-		paranoid_fclose(fout);
-		if (j > 0) {
-			log_to_screen("%d files listed in %s/changed.files have been deleted since backup was made\nand are referenced in %s", j, bkpinfo->tmpdir,tmp1);
-		}
-		paranoid_free(tmp1);
 		if (filelist->entries >= ARBITRARY_MAXIMUM) {
 			log_to_screen("Arbitrary limits suck, man!");
 			paranoid_free(tmp);
