@@ -7,7 +7,7 @@ copyright : (C) 2002 Mondo  Hugo Rabson
 email     : Hugo Rabson <hugorabson@msn.com>
 edited by : by Stan Benoit 4/2002
 email     : troff@nakedsoul.org
-cvsid     : $Id: mondo-prep.c 2037 2008-10-06 22:12:01Z bruno $
+cvsid     : $Id: mondo-prep.c 1320 2007-04-16 17:24:58Z bruno $
 ***************************************************************************/
 
 /***************************************************************************
@@ -153,7 +153,7 @@ cvsid     : $Id: mondo-prep.c 2037 2008-10-06 22:12:01Z bruno $
  */
 
 
-#include "my-stuff.h"
+#include "../common/my-stuff.h"
 #include "../common/mondostructures.h"
 #include "mondoprep.h"
 #include "../common/libmondo.h"
@@ -180,7 +180,7 @@ cvsid     : $Id: mondo-prep.c 2037 2008-10-06 22:12:01Z bruno $
 #define ARCHIVES_PATH MNT_CDROM"/archives"
 #define MONDO_WAS_HERE "MONDOWOZEREMONDOWOZEREMONDOWOZEREhahahaMOJOJOJO"
 
-//static char cvsid[] = "$Id: mondo-prep.c 2037 2008-10-06 22:12:01Z bruno $";
+//static char cvsid[] = "$Id: mondo-prep.c 1320 2007-04-16 17:24:58Z bruno $";
 
 extern char *g_mountlist_fname;
 extern long g_current_progress, g_maximum_progress;
@@ -190,6 +190,7 @@ extern bool g_text_mode;
 extern void pause_for_N_seconds(int, char *);
 extern char *MONDO_LOGFILE;
 
+
 FILE *g_fprep = NULL;
 
 
@@ -397,7 +398,7 @@ int do_my_funky_lvm_stuff(bool just_erase_existing_volumes,
 		strcpy(pvscan_sz, "lvm pvscan");
 		strcpy(vgcreate_sz, "lvm vgcreate");
 		strcpy(vgchange_sz, "lvm vgchange");
-		strcpy(vgremove_sz, "lvm vgremove -f");
+		strcpy(vgremove_sz, "lvm vgremove");
 	} else {
 		strcpy(lvscan_sz, "lvscan");
 		strcpy(lvremove_sz, "lvremove");
@@ -2210,10 +2211,6 @@ int set_partition_type(FILE * pout_to_fdisk, const char *drive, int partno,
 		strcpy(partcode, "83");
 	} else if (strcmp(format, "minix") == 0) {
 		strcpy(partcode, "81");
-	} else if (strcmp(format, "vmfs3") == 0) {
-		strcpy(partcode, "fb");
-	} else if (strcmp(format, "vmkcore") == 0) {
-		strcpy(partcode, "fc");
 	} else if (strcmp(format, "raid") == 0) {
 		strcpy(partcode, "fd");
 	} else if ((strcmp(format, "ufs") == 0)
@@ -2531,17 +2528,9 @@ int which_format_command_do_i_need(char *format, char *program)
 	} else if (strcmp(format, "jfs") == 0) {
 		strcpy(program, "mkfs.jfs");
 	} else if (strcmp(format, "ext3") == 0) {
-		strcpy(program, "mkfs -t ext3 -F -q");
+		strcpy(program, "mkfs -t ext2 -F -j -q");
 	} else if (strcmp(format, "minix") == 0) {
 		strcpy(program, "mkfs.minix");
-	} else if (strcmp(format, "vmfs") == 0) {
-		strcpy(program, "mkfs -t vmfs");
-	} else if (strcmp(format, "ocfs2") == 0) {
-		/*
-		 * For existing OCFS2 volumes, mkfs.ocfs2 ensures the volume is not mounted on any node in the cluster before formatting. For that to work, mkfs.ocfs2 expects the O2CB cluster service to be running. Specify this option to disable this check.
-		 *
-		 */
-		strcpy(program, "mkfs -t ocfs2 -F");
 #endif
 	} else if (strcmp(format, "ext2") == 0) {
 		strcpy(program, "mke2fs -F -q");
@@ -2561,6 +2550,48 @@ int which_format_command_do_i_need(char *format, char *program)
 }
 
 
+/**
+ * Calculate the probable size of @p drive_name by adding up sizes in
+ * @p mountlist.
+ * @param mountlist The mountlist to use to calculate the size.
+ * @param drive_name The drive to calculate the original size of.
+ * @return The size of @p drive_name in kilobytes.
+ */
+long calc_orig_size_of_drive_from_mountlist(struct mountlist_itself
+											*mountlist, char *drive_name)
+{
+	/** long ************************************************************/
+	long original_size_of_drive;
+
+	/** int *************************************************************/
+	int partno;
+
+	/** buffers *********************************************************/
+	char *tmp;
+
+	/** end *************************************************************/
+
+	malloc_string(tmp);
+	assert(mountlist != NULL);
+	assert_string_is_neither_NULL_nor_zerolength(drive_name);
+
+	for (original_size_of_drive = 0, partno = 0;
+		 partno < mountlist->entries; partno++) {
+		if (strncmp
+			(mountlist->el[partno].device, drive_name,
+			 strlen(drive_name)) == 0) {
+			original_size_of_drive += mountlist->el[partno].size;
+		} else {
+			sprintf(tmp, "Skipping %s", mountlist->el[partno].device);
+//            log_to_screen(tmp);
+		}
+	}
+	original_size_of_drive = original_size_of_drive / 1024;
+	paranoid_free(tmp);
+	return (original_size_of_drive);
+}
+
+
 /**
  * Resize a drive's entries in @p mountlist proportionately to fit its new size.
  * There are a few problems with this function:
@@ -2590,8 +2621,8 @@ void resize_drive_proportionately_to_suit_new_drives(struct mountlist_itself
 
 	/** long *************************************************************/
 	long newsizL;
-	long current_size_of_drive = 0L;
-	long original_size_of_drive = 0L;
+	long current_size_of_drive = 0;
+	long original_size_of_drive = 0;
 	long final_size;			/* all in Megabytes */
 	struct mountlist_reference *drivemntlist;
 
